@@ -182,7 +182,7 @@ bool ArgumentReader::read(ArgumentVector &args)
     Argument *lastArgInLevel = nullptr;
     vector<const char *> *values = nullptr;
 
-    // iterate through all argument denotations; loop might exit earlier when an denotation is unknown
+    // iterate through all argument denotations; loop might exit earlier when a denotation is unknown
     while (argv != end) {
         // check whether there are still values to read
         if (values && ((lastArgInLevel->requiredValueCount() != Argument::varValueCount) || (lastArgInLevel->flags() & Argument::Flags::Greedy))
@@ -361,6 +361,9 @@ bool ArgumentReader::read(ArgumentVector &args)
             ++parser.m_actualArgc;
             lastArg = lastArgInLevel = matchingArg;
             argDenotation = nullptr;
+            if ((values->size() < matchingArg->requiredValueCount()) && (matchingArg->flags() & Argument::Flags::Greedy)) {
+                continue;
+            }
             read(lastArg->m_subArgs);
             argDenotation = nullptr;
             continue;
@@ -1320,6 +1323,25 @@ string ArgumentParser::findSuggestions(int argc, const char *const *argv, unsign
     return suggestionStr;
 }
 
+/*!
+ * \brief Returns a copy of \a escaped with escaping characters removed.
+ */
+static std::string unescape(std::string_view escaped)
+{
+    auto unescaped = std::string();
+    auto onEscaping = false;
+    unescaped.reserve(escaped.size());
+    for (const auto c : escaped) {
+        if (!onEscaping && c == '\\') {
+            onEscaping = true;
+        } else {
+            unescaped += c;
+            onEscaping = false;
+        }
+    }
+    return unescaped;
+}
+
 /*!
  * \brief Prints the bash completion for the specified arguments and the specified \a lastPath.
  * \remarks Arguments must have been parsed before with readSpecifiedArgs(). When calling this method, completionMode must
@@ -1505,23 +1527,13 @@ void ArgumentParser::printBashCompletion(int argc, const char *const *argv, unsi
     }
     // -> completions for files and dirs
     // -> if there's already an "opening", determine the dir part and the file part
-    string actualDir, actualFile;
-    bool haveFileOrDirCompletions = false;
+    auto actualDir = std::string(), actualFile = std::string();
+    auto haveFileOrDirCompletions = false;
     if (argc && currentWordIndex == completionInfo.lastSpecifiedArgIndex && opening) {
-        // the "opening" might contain escaped characters which need to be unescaped first (let's hope this covers all possible escapings)
-        string unescapedOpening(opening);
-        findAndReplace<string>(unescapedOpening, "\\ ", " ");
-        findAndReplace<string>(unescapedOpening, "\\,", ",");
-        findAndReplace<string>(unescapedOpening, "\\[", "[");
-        findAndReplace<string>(unescapedOpening, "\\]", "]");
-        findAndReplace<string>(unescapedOpening, "\\!", "!");
-        findAndReplace<string>(unescapedOpening, "\\#", "#");
-        findAndReplace<string>(unescapedOpening, "\\$", "$");
-        findAndReplace<string>(unescapedOpening, "\\'", "'");
-        findAndReplace<string>(unescapedOpening, "\\\"", "\"");
-        findAndReplace<string>(unescapedOpening, "\\\\", "\\");
+        // the "opening" might contain escaped characters which need to be unescaped first
+        const auto unescapedOpening = unescape(opening);
         // determine the "directory" part
-        string dir = directory(unescapedOpening);
+        auto dir = directory(unescapedOpening);
         if (dir.empty()) {
             actualDir = ".";
         } else {
@@ -1531,17 +1543,17 @@ void ArgumentParser::printBashCompletion(int argc, const char *const *argv, unsi
             if (dir.size() > 1 && (dir[dir.size() - 2] == '\"' || dir[dir.size() - 2] == '\'')) {
                 dir.erase(dir.size() - 2, 1);
             }
-            actualDir = move(dir);
+            actualDir = std::move(dir);
         }
         // determine the "file" part
-        string file = fileName(unescapedOpening);
+        auto file = fileName(unescapedOpening);
         if (file[0] == '\"' || file[0] == '\'') {
             file.erase(0, 1);
         }
         if (file.size() > 1 && (file[file.size() - 2] == '\"' || file[file.size() - 2] == '\'')) {
             file.erase(file.size() - 2, 1);
         }
-        actualFile = move(file);
+        actualFile = std::move(file);
     }
 
     // -> completion for files and dirs
