@@ -7,6 +7,7 @@
 #include <functional>
 #include <initializer_list>
 #include <limits>
+#include <tuple>
 #include <vector>
 #ifdef CPP_UTILITIES_DEBUG_BUILD
 #include <cassert>
@@ -530,7 +531,7 @@ inline const char *Argument::name() const
  */
 inline void Argument::setName(const char *name)
 {
-#ifdef CPP_UTILITIES_DEBUG_BUILD
+#if defined(CPP_UTILITIES_DEBUG_BUILD) && !defined(_MSC_VER)
     if (name && *name) {
         assert(*name != '-');
         for (const char *c = name; *c; ++c) {
