@@ -2,10 +2,13 @@
 
 #ifndef CPP_UTILITIES_NO_THREAD_LOCAL
 #include "../feature_detection/features.h"
-#else
+#endif
+
+#ifndef CPP_UTILITIES_THREAD_LOCAL
 #define CPP_UTILITIES_THREAD_LOCAL
 #endif
 
+#include <algorithm>
 #include <cmath>
 #include <cstdlib>
 #include <iomanip>
@@ -18,6 +21,13 @@
 
 #ifdef PLATFORM_WINDOWS
 #include <windows.h>
+// note: The windows header seriously defines a macro called "max" breaking the (common) use
+// of std::numeric_limits in the subsequent code. So we need to undefine this macro. Note that
+// this is not the case using mingw-w64 but it is happening with windows.h from Windows Kits
+// version 10.0.22000.0 via Visual Studio 2022.
+#ifdef max
+#undef max
+#endif
 #endif
 
 using namespace std;
@@ -194,22 +204,25 @@ StringData convertUtf8ToLatin1(const char *inputBuffer, std::size_t inputBufferS
 #ifdef PLATFORM_WINDOWS
 /*!
  * \brief Converts the specified multi-byte string (assumed to be UTF-8) to a wide string using the WinAPI.
- * \remarks Only available under Windows.
+ * \remarks
+ * - Only available under Windows.
+ * - If \a inputBuffer exceeds std::numeric_limits<int>::max() it will be truncated.
  */
 std::wstring convertMultiByteToWide(std::error_code &ec, std::string_view inputBuffer)
 {
     // calculate required size
     auto widePath = std::wstring();
-    auto size = MultiByteToWideChar(CP_UTF8, 0, inputBuffer.data(), inputBuffer.size(), nullptr, 0);
+    auto bufferSize = static_cast<int>(std::clamp<std::size_t>(inputBuffer.size(), 0, std::numeric_limits<int>::max()));
+    auto size = MultiByteToWideChar(CP_UTF8, 0, inputBuffer.data(), bufferSize, nullptr, 0);
     if (size <= 0) {
-        ec = std::error_code(GetLastError(), std::system_category());
+        ec = std::error_code(static_cast<int>(GetLastError()), std::system_category());
         return widePath;
     }
     // do the actual conversion
     widePath.resize(static_cast<std::wstring::size_type>(size));
-    size = MultiByteToWideChar(CP_UTF8, 0, inputBuffer.data(), inputBuffer.size(), widePath.data(), size);
+    size = MultiByteToWideChar(CP_UTF8, 0, inputBuffer.data(), bufferSize, widePath.data(), size);
     if (size <= 0) {
-        ec = std::error_code(GetLastError(), std::system_category());
+        ec = std::error_code(static_cast<int>(GetLastError()), std::system_category());
         widePath.clear();
     }
     return widePath;
@@ -227,14 +240,14 @@ WideStringData convertMultiByteToWide(std::error_code &ec, const char *inputBuff
     WideStringData widePath;
     widePath.second = MultiByteToWideChar(CP_UTF8, 0, inputBuffer, inputBufferSize, nullptr, 0);
     if (widePath.second <= 0) {
-        ec = std::error_code(GetLastError(), std::system_category());
+        ec = std::error_code(static_cast<int>(GetLastError()), std::system_category());
         return widePath;
     }
     // do the actual conversion
     widePath.first = make_unique<wchar_t[]>(static_cast<size_t>(widePath.second));
     widePath.second = MultiByteToWideChar(CP_UTF8, 0, inputBuffer, inputBufferSize, widePath.first.get(), widePath.second);
     if (widePath.second <= 0) {
-        ec = std::error_code(GetLastError(), std::system_category());
+        ec = std::error_code(static_cast<int>(GetLastError()), std::system_category());
         widePath.first.reset();
     }
     return widePath;
@@ -246,8 +259,8 @@ WideStringData convertMultiByteToWide(std::error_code &ec, const char *inputBuff
  */
 WideStringData convertMultiByteToWide(std::error_code &ec, const std::string &inputBuffer)
 {
-    return convertMultiByteToWide(
-        ec, inputBuffer.data(), inputBuffer.size() < (numeric_limits<int>::max() - 1) ? static_cast<int>(inputBuffer.size() + 1) : -1);
+    return convertMultiByteToWide(ec, inputBuffer.data(),
+        inputBuffer.size() < static_cast<std::size_t>(std::numeric_limits<int>::max() - 1) ? static_cast<int>(inputBuffer.size() + 1) : -1);
 }
 
 /*!
@@ -401,22 +414,23 @@ string encodeBase64(const std::uint8_t *data, std::uint32_t dataSize)
  * \throw Throws a ConversionException if the specified string is no valid Base64.
  * \sa [RFC 4648](http://www.ietf.org/rfc/rfc4648.txt)
  */
-pair<unique_ptr<std::uint8_t[]>, std::uint32_t> decodeBase64(const char *encodedStr, const std::uint32_t strSize)
+std::pair<unique_ptr<std::uint8_t[]>, std::uint32_t> decodeBase64(const char *encodedStr, const std::uint32_t strSize)
 {
+    if (!strSize) {
+        return std::make_pair(std::make_unique<std::uint8_t[]>(0), 0); // early return to prevent clazy warning
+    }
     if (strSize % 4) {
         throw ConversionException("invalid size of base64");
     }
     std::uint32_t decodedSize = (strSize / 4) * 3;
     const char *const end = encodedStr + strSize;
-    if (strSize) {
-        if (*(end - 1) == base64Pad) {
-            --decodedSize;
-        }
-        if (*(end - 2) == base64Pad) {
-            --decodedSize;
-        }
+    if (*(end - 1) == base64Pad) {
+        --decodedSize;
     }
-    auto buffer = make_unique<std::uint8_t[]>(decodedSize);
+    if (*(end - 2) == base64Pad) {
+        --decodedSize;
+    }
+    auto buffer = std::make_unique<std::uint8_t[]>(decodedSize);
     auto *iter = buffer.get() - 1;
     while (encodedStr < end) {
         std::int32_t temp = 0;
@@ -437,10 +451,10 @@ pair<unique_ptr<std::uint8_t[]>, std::uint32_t> decodeBase64(const char *encoded
                 case 1:
                     *++iter = static_cast<std::uint8_t>((temp >> 16) & 0xFF);
                     *++iter = static_cast<std::uint8_t>((temp >> 8) & 0xFF);
-                    return make_pair(move(buffer), decodedSize);
+                    return std::make_pair(std::move(buffer), decodedSize);
                 case 2:
                     *++iter = static_cast<std::uint8_t>((temp >> 10) & 0xFF);
-                    return make_pair(move(buffer), decodedSize);
+                    return std::make_pair(std::move(buffer), decodedSize);
                 default:
                     throw ConversionException("invalid padding in base64");
                 }
@@ -452,6 +466,6 @@ pair<unique_ptr<std::uint8_t[]>, std::uint32_t> decodeBase64(const char *encoded
         *++iter = static_cast<std::uint8_t>((temp >> 8) & 0xFF);
         *++iter = static_cast<std::uint8_t>(temp & 0xFF);
     }
-    return make_pair(move(buffer), decodedSize);
+    return std::make_pair(std::move(buffer), decodedSize);
 }
 } // namespace CppUtilities
