@@ -6,15 +6,14 @@
 namespace CppUtilities {
 
 /*!
- * \brief The IsFlagEnumClass class is used to decide whether to enable operations for flag enums for \tp T.
+ * \brief The IsFlagEnumClass class is used to decide whether to enable operations for flag enums for \tparam T.
  * \remarks This class is still experimental and might be changed or removed in future minior releases.
  */
-template <typename T> struct IsFlagEnumClass : public Traits::Bool<false> {
-};
+template <typename T> struct IsFlagEnumClass : public Traits::Bool<false> {};
 
 // clang-format off
 /*!
- * \def The CPP_UTILITIES_MARK_FLAG_ENUM_CLASS macro enables flag enum operators for \a EnumClassType within namespace \a Namespace.
+ * \brief The \def CPP_UTILITIES_MARK_FLAG_ENUM_CLASS macro enables flag enum operators for \a EnumClassType within namespace \a Namespace.
  * \remarks
  * - Must be used outside a namespace.
  * - This macro is still experimental and might be changed or removed in future minior releases.
