@@ -21,6 +21,7 @@
 
 #include <cstdlib>
 #include <cstring>
+#include <regex>
 
 #ifdef PLATFORM_WINDOWS
 #include <windows.h>
@@ -109,7 +110,7 @@ void ArgumentParserTests::testArgument()
 #endif
     ArgumentOccurrence occurrence(0, vector<Argument *>(), nullptr);
     occurrence.values.emplace_back("bar");
-    argument.m_occurrences.emplace_back(move(occurrence));
+    argument.m_occurrences.emplace_back(std::move(occurrence));
     CPPUNIT_ASSERT_EQUAL("bar"s, string(argument.firstValue()));
 }
 
@@ -815,30 +816,30 @@ void ArgumentParserTests::testHelp()
     // parse args and assert output
     const char *const argv[] = { "app", "-h" };
     {
-        const OutputCheck c("\e[1m" APP_NAME ", version " APP_VERSION "\n"
+        const OutputCheck c("\033[1m" APP_NAME ", version " APP_VERSION "\n"
                             "\n"
-                            "\e[0m" APP_DESCRIPTION "\n"
+                            "\033[0m" APP_DESCRIPTION "\n"
                             "\n"
                             "Available operations:\n"
-                            "\e[1mverbose, -v\e[0m\n"
+                            "\033[1mverbose, -v\033[0m\n"
                             "  be verbose\n"
                             "  example: actually not an operation\n"
                             "\n"
                             "Available top-level options:\n"
-                            "\e[1m--files, -f\e[0m\n"
+                            "\033[1m--files, -f\033[0m\n"
                             "  specifies the path of the file(s) to be opened\n"
-                            "  \e[1m--sub\e[0m\n"
+                            "  \033[1m--sub\033[0m\n"
                             "    sub arg\n"
                             "    particularities: mandatory if parent argument is present\n"
-                            "    \e[1m--nested-sub\e[0m [value1] [value2] ...\n"
+                            "    \033[1m--nested-sub\033[0m [value1] [value2] ...\n"
                             "      nested sub arg\n"
                             "    example: sub arg example\n"
                             "\n"
-                            "\e[1m--env\e[0m [file] [value 2]\n"
+                            "\033[1m--env\033[0m [file] [value 2]\n"
                             "  env\n"
                             "  default environment variable: FILES\n"
                             "\n"
-                            "\e[1m--no-color\e[0m\n"
+                            "\033[1m--no-color\033[0m\n"
                             "  disables formatted/colorized output\n"
                             "  default environment variable: ENABLE_ESCAPE_CODES\n"
                             "\n"
@@ -981,8 +982,9 @@ void ArgumentParserTests::testValueConversion()
         occurrence.convertValues<int>();
         CPPUNIT_FAIL("Expected exception");
     } catch (const ParseError &failure) {
-        CPPUNIT_ASSERT_EQUAL(
-            "Conversion of top-level value \"foo\" to type \"i\" failed: The character \"f\" is no valid digit."s, string(failure.what()));
+        TESTUTILS_ASSERT_LIKE_FLAGS("conversion error of top-level value",
+            "Conversion of top-level value \"foo\" to type \".*\" failed: The character \"f\" is no valid digit."s, std::regex::extended,
+            std::string(failure.what()));
     }
     occurrence.path = { &arg };
     try {
@@ -995,7 +997,8 @@ void ArgumentParserTests::testValueConversion()
         occurrence.convertValues<int>();
         CPPUNIT_FAIL("Expected exception");
     } catch (const ParseError &failure) {
-        CPPUNIT_ASSERT_EQUAL("Conversion of value \"foo\" (for argument --test) to type \"i\" failed: The character \"f\" is no valid digit."s,
-            string(failure.what()));
+        TESTUTILS_ASSERT_LIKE_FLAGS("conversion error of argument value",
+            "Conversion of value \"foo\" \\(for argument --test\\) to type \".*\" failed: The character \"f\" is no valid digit."s,
+            std::regex::extended, std::string(failure.what()));
     }
 }
