@@ -27,6 +27,9 @@ static_assert(toNormalInt(383) == 255, "toNormalInt()");
 static_assert(swapOrder(static_cast<std::uint16_t>(0xABCD)) == 0xCDAB, "swapOrder(std::uint16_t)");
 static_assert(swapOrder(static_cast<std::uint32_t>(0xABCDEF12u)) == 0x12EFCDABu, "swapOrder(std::uint32_t)");
 static_assert(swapOrder(static_cast<std::uint64_t>(0xABCDEF1234567890ul)) == 0x9078563412EFCDABul, "swapOrder(std::uint64_t)");
+static_assert(swapOrder(static_cast<std::int16_t>(0xABCD)) == static_cast<std::int16_t>(0xCDAB), "swapOrder(std::int16_t)");
+static_assert(swapOrder(static_cast<std::int32_t>(0xABCDEF12)) == 0x12EFCDAB, "swapOrder(std::int32_t)");
+static_assert(swapOrder(static_cast<std::int64_t>(0xABCDEF1234567890l)) == static_cast<std::int64_t>(0x9078563412EFCDABl), "swapOrder(std::int64_t)");
 
 /*!
  * \brief The ConversionTests class tests classes and functions provided by the files inside the conversion directory.
@@ -116,13 +119,13 @@ void ConversionTests::testConversion(
     stringstream msg;
     msg << message << '(' << hex << '0' << 'x' << random << ')';
     vice(random, m_buff);
-    CPPUNIT_ASSERT_MESSAGE(msg.str(), versa(m_buff) == random);
+    CPPUNIT_ASSERT_EQUAL_MESSAGE(msg.str(), random, versa(m_buff));
 }
 
 #define TEST_TYPE(endianness, function) decltype(endianness::function(m_buff))
 
 #define TEST_CONVERSION(function, endianness)                                                                                                        \
-    testConversion<TEST_TYPE(endianness, function)>("testing " #function,                                                                            \
+    testConversion<TEST_TYPE(endianness, function)>("testing " #endianness "::" #function,                                                           \
         static_cast<void (*)(TEST_TYPE(endianness, function), char *)>(&endianness::getBytes), endianness::function,                                 \
         numeric_limits<TEST_TYPE(endianness, function)>::min(), numeric_limits<TEST_TYPE(endianness, function)>::max())
 
@@ -131,8 +134,8 @@ void ConversionTests::testConversion(
 #define TEST_LE_CONVERSION(function) TEST_CONVERSION(function, LE)
 
 #define TEST_CUSTOM_CONVERSION(vice, versa, endianness, min, max)                                                                                    \
-    testConversion<TEST_TYPE(endianness, versa)>(                                                                                                    \
-        "testing " #versa, static_cast<void (*)(TEST_TYPE(endianness, versa), char *)>(&endianness::vice), endianness::versa, min, max)
+    testConversion<TEST_TYPE(endianness, versa)>("testing " #versa " (" #endianness ")",                                                             \
+        static_cast<void (*)(TEST_TYPE(endianness, versa), char *)>(&endianness::vice), endianness::versa, min, max)
 
 /*!
  * \brief Tests most important binary conversions.
@@ -363,10 +366,10 @@ void ConversionTests::testStringConversions()
     CPPUNIT_ASSERT_EQUAL("foo"s, truncateTest);
 
     // encodeBase64() / decodeBase64() with random data
-    uniform_int_distribution<std::uint8_t> randomDistChar;
+    std::uniform_int_distribution<int> randomDistChar;
     std::uint8_t originalBase64Data[4047];
     for (std::uint8_t &c : originalBase64Data) {
-        c = randomDistChar(m_randomEngine);
+        c = static_cast<std::uint8_t>(randomDistChar(m_randomEngine) & 0xFF);
     }
     auto encodedBase64Data = encodeBase64(originalBase64Data, sizeof(originalBase64Data));
     auto decodedBase64Data = decodeBase64(encodedBase64Data.data(), static_cast<std::uint32_t>(encodedBase64Data.size()));
