@@ -27,6 +27,19 @@
 #include <unistd.h>
 #endif
 
+#ifdef CPP_UTILITIES_BOOST_PROCESS
+#include <boost/asio/buffers_iterator.hpp>
+#include <boost/asio/io_context.hpp>
+#include <boost/asio/streambuf.hpp>
+#include <boost/process/async.hpp>
+#include <boost/process/child.hpp>
+#include <boost/process/env.hpp>
+#include <boost/process/environment.hpp>
+#include <boost/process/group.hpp>
+#include <boost/process/io.hpp>
+#include <boost/process/search_path.hpp>
+#endif
+
 #ifdef PLATFORM_WINDOWS
 #include <windows.h>
 #endif
@@ -40,7 +53,7 @@ using namespace CppUtilities::EscapeCodes;
 namespace CppUtilities {
 
 /// \cond
-bool fileSystemItemExists(const string &path)
+static bool fileSystemItemExists(const string &path)
 {
 #ifdef PLATFORM_UNIX
     struct stat res;
@@ -55,7 +68,7 @@ bool fileSystemItemExists(const string &path)
 #endif
 }
 
-bool fileExists(const string &path)
+static bool fileExists(const string &path)
 {
 #ifdef PLATFORM_UNIX
     struct stat res;
@@ -70,7 +83,7 @@ bool fileExists(const string &path)
 #endif
 }
 
-bool dirExists(const string &path)
+static bool dirExists(const string &path)
 {
 #ifdef PLATFORM_UNIX
     struct stat res;
@@ -85,7 +98,7 @@ bool dirExists(const string &path)
 #endif
 }
 
-bool makeDir(const string &path)
+static bool makeDir(const string &path)
 {
 #ifdef PLATFORM_UNIX
     return mkdir(path.data(), S_IRWXU | S_IRWXG | S_IROTH | S_IXOTH) == 0;
@@ -174,11 +187,12 @@ TestApplication::TestApplication(int argc, const char *const *argv)
     // -> read TEST_FILE_PATH environment variable
     bool hasTestFilePathFromEnv;
     if (auto testFilePathFromEnv = readTestfilePathFromEnv(); (hasTestFilePathFromEnv = !testFilePathFromEnv.empty())) {
-        m_testFilesPaths.emplace_back(move(testFilePathFromEnv));
+        m_testFilesPaths.emplace_back(std::move(testFilePathFromEnv));
     }
     // -> find source directory
     if (auto testFilePathFromSrcDirRef = readTestfilePathFromSrcRef(); !testFilePathFromSrcDirRef.empty()) {
-        m_testFilesPaths.emplace_back(move(testFilePathFromSrcDirRef));
+        m_testFilesPaths.insert(m_testFilesPaths.end(), std::make_move_iterator(testFilePathFromSrcDirRef.begin()),
+            std::make_move_iterator(testFilePathFromSrcDirRef.end()));
     }
     // -> try testfiles directory in working directory
     m_testFilesPaths.emplace_back("./testfiles/");
@@ -390,7 +404,16 @@ string TestApplication::workingCopyPathAs(
     return workingCopyPath;
 }
 
-#ifdef PLATFORM_UNIX
+#ifdef CPP_UTILITIES_HAS_EXEC_APP
+
+#if defined(CPP_UTILITIES_BOOST_PROCESS)
+inline static std::string streambufToString(boost::asio::streambuf &buf)
+{
+    const auto begin = boost::asio::buffers_begin(buf.data());
+    return std::string(begin, begin + static_cast<std::ptrdiff_t>(buf.size()));
+}
+#endif
+
 /*!
  * \brief Executes an application with the specified \a args.
  * \remarks Provides internal implementation of execApp() and execHelperApp().
@@ -410,6 +433,48 @@ static int execAppInternal(const char *appPath, const char *const *args, std::st
         cout << endl;
     }
 
+#if defined(CPP_UTILITIES_BOOST_PROCESS)
+    auto path = enableSearchPath ? boost::process::search_path(appPath) : boost::process::filesystem::path(appPath);
+    auto ctx = boost::asio::io_context();
+    auto group = boost::process::group();
+    auto argsAsVector =
+#if defined(PLATFORM_WINDOWS)
+        std::vector<std::wstring>();
+#else
+        std::vector<std::string>();
+#endif
+    if (*args) {
+        for (const char *const *arg = args + 1; *arg; ++arg) {
+#if defined(PLATFORM_WINDOWS)
+            auto ec = std::error_code();
+            argsAsVector.emplace_back(convertMultiByteToWide(ec, std::string_view(*arg)));
+            if (ec) {
+                throw std::runtime_error(argsToString("unable to convert arg \"", *arg, "\" to wide string"));
+            }
+#else
+            argsAsVector.emplace_back(*arg);
+#endif
+        }
+    }
+    auto outputBuffer = boost::asio::streambuf(), errorBuffer = boost::asio::streambuf();
+    auto env = boost::process::environment(boost::this_process::environment());
+    if (!newProfilingPath.empty()) {
+        env["LLVM_PROFILE_FILE"] = newProfilingPath;
+    }
+    auto child
+        = boost::process::child(ctx, group, path, argsAsVector, env, boost::process::std_out > outputBuffer, boost::process::std_err > errorBuffer);
+    if (timeout > 0) {
+        ctx.run_for(std::chrono::milliseconds(timeout));
+    } else {
+        ctx.run();
+    }
+    output = streambufToString(outputBuffer);
+    errors = streambufToString(errorBuffer);
+    child.wait();
+    group.wait();
+    return child.exit_code();
+
+#elif defined(PLATFORM_UNIX)
     // create pipes
     int coutPipes[2], cerrPipes[2];
     pipe(coutPipes);
@@ -503,6 +568,10 @@ static int execAppInternal(const char *appPath, const char *const *args, std::st
         cerr << Phrases::Error << "Unable to execute \"" << appPath << "\": execv() failed" << Phrases::EndFlush;
         exit(-101);
     }
+
+#else
+    throw std::runtime_error("lauching test applications is not supported on this platform");
+#endif
 }
 
 /*!
@@ -593,7 +662,7 @@ int execHelperAppInSearchPath(
 {
     return execAppInternal(appName, args, output, errors, suppressLogging, timeout, string(), true);
 }
-#endif // PLATFORM_UNIX
+#endif
 
 /*!
  * \brief Reads the path of the test file directory from the environment variable TEST_FILE_PATH.
@@ -612,10 +681,11 @@ string TestApplication::readTestfilePathFromEnv()
  * \remarks That file is supposed to contain the path the the source directory. It is supposed to be stored by the build system in the
  *          same directory as the test executable. The CMake modules contained of these utilities ensure that's the case.
  */
-string TestApplication::readTestfilePathFromSrcRef()
+std::vector<std::string> TestApplication::readTestfilePathFromSrcRef()
 {
     // find the path of the current executable on platforms supporting "/proc/self/exe"; otherwise assume the current working directory
     // is the executable path
+    auto res = std::vector<std::string>();
     auto binaryPath = std::string();
 #if defined(CPP_UTILITIES_USE_STANDARD_FILESYSTEM) && defined(PLATFORM_UNIX)
     try {
@@ -628,25 +698,29 @@ string TestApplication::readTestfilePathFromSrcRef()
     const auto srcdirrefPath = binaryPath + "srcdirref";
     try {
         // read "srcdirref" file which should contain the path of the source directory
-        auto srcDirContent(readFile(srcdirrefPath, 2 * 1024));
+        const auto srcDirContent = readFile(srcdirrefPath, 2 * 1024);
         if (srcDirContent.empty()) {
             cerr << Phrases::Warning << "The file \"srcdirref\" is empty." << Phrases::EndFlush;
-            return string();
+            return res;
         }
-        srcDirContent += "/testfiles/";
 
-        // check whether the referenced source directory contains a "testfiles" directory
-        if (!dirExists(srcDirContent)) {
-            cerr << Phrases::Warning
-                 << "The source directory referenced by the file \"srcdirref\" does not contain a \"testfiles\" directory or does not exist."
-                 << Phrases::End << "Referenced source directory: " << srcDirContent << endl;
-            return string();
+        // check whether the referenced source directories contain a "testfiles" directory
+        const auto srcPaths = splitStringSimple<std::vector<std::string_view>>(srcDirContent, "\n");
+        for (const auto &srcPath : srcPaths) {
+            auto testfilesPath = argsToString(srcPath, "/testfiles/");
+            if (dirExists(testfilesPath)) {
+                res.emplace_back(std::move(testfilesPath));
+            } else {
+                cerr << Phrases::Warning
+                     << "The source directory referenced by the file \"srcdirref\" does not contain a \"testfiles\" directory or does not exist."
+                     << Phrases::End << "Referenced source directory: " << testfilesPath << endl;
+            }
         }
-        return srcDirContent;
+        return res;
 
     } catch (const std::ios_base::failure &e) {
         cerr << Phrases::Warning << "The file \"" << srcdirrefPath << "\" can not be opened: " << e.what() << Phrases::EndFlush;
     }
-    return string();
+    return res;
 }
 } // namespace CppUtilities
